@@ -26,11 +26,12 @@
  * around the fetcher specific methods.
  *
  * Active fetches are held in the circular linked list ::fetch_ring. There may
- * be at most ::option_max_fetchers_per_host active requests per Host: header.
- * There may be at most ::option_max_fetchers active requests overall. Inactive
+ * be at most nsoption max_fetchers_per_host active requests per Host: header.
+ * There may be at most nsoption max_fetchers active requests overall. Inactive
  * fetches are stored in the ::queue_ring waiting for use.
  */
 
+#include <stdlib.h>
 #include <assert.h>
 #include <errno.h>
 #include <stdbool.h>
@@ -38,7 +39,6 @@
 #include <strings.h>
 #include <time.h>
 #include <libwapcaplet/libwapcaplet.h>
-#include <curl/curl.h>
 
 #include "utils/config.h"
 #include "utils/corestrings.h"
@@ -46,9 +46,8 @@
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/nsurl.h"
-#include "utils/utils.h"
 #include "utils/ring.h"
-#include "desktop/gui_misc.h"
+#include "netsurf/misc.h"
 #include "desktop/gui_internal.h"
 
 #include "content/fetch.h"
@@ -61,16 +60,6 @@
 #include "javascript/fetcher.h"
 #include "content/urldb.h"
 
-/* Define this to turn on verbose fetch logging */
-#undef DEBUG_FETCH_VERBOSE
-
-/** Verbose fetcher logging */
-#ifdef DEBUG_FETCH_VERBOSE
-#define FETCH_LOG(x) LOG(x)
-#else
-#define FETCH_LOG(x)
-#endif
-
 /** The maximum number of fetchers that can be added */
 #define MAX_FETCHERS 10
 
@@ -159,8 +148,10 @@ static int get_fetcher_for_scheme(lwc_string *scheme)
 static bool fetch_dispatch_job(struct fetch *fetch)
 {
 	RING_REMOVE(queue_ring, fetch);
-	FETCH_LOG(("Attempting to start fetch %p, fetcher %p, url %s", fetch,
-	     fetch->fetcher_handle, nsurl_access(fetch->url)));
+	NSLOG(fetch, DEBUG,
+	      "Attempting to start fetch %p, fetcher %p, url %s", fetch,
+	      fetch->fetcher_handle,
+	      nsurl_access(fetch->url));
 
 	if (!fetchers[fetch->fetcherd].ops.start(fetch->fetcher_handle)) {
 		RING_INSERT(queue_ring, fetch); /* Put it back on the end of the queue */
@@ -211,25 +202,25 @@ static bool fetch_choose_and_dispatch(void)
 
 static void dump_rings(void)
 {
-#ifdef DEBUG_FETCH_VERBOSE
 	struct fetch *q;
 	struct fetch *f;
 
 	q = queue_ring;
 	if (q) {
 		do {
-			LOG(("queue_ring: %s", nsurl_access(q->url)));
+			NSLOG(fetch, DEBUG, "queue_ring: %s",
+			      nsurl_access(q->url));
 			q = q->r_next;
 		} while (q != queue_ring);
 	}
 	f = fetch_ring;
 	if (f) {
 		do {
-			LOG(("fetch_ring: %s", nsurl_access(f->url)));
+			NSLOG(fetch, DEBUG, "fetch_ring: %s",
+			      nsurl_access(f->url));
 			f = f->r_next;
 		} while (f != fetch_ring);
 	}
-#endif
 }
 
 /**
@@ -245,7 +236,10 @@ static bool fetch_dispatch_jobs(void)
 	RING_GETSIZE(struct fetch, queue_ring, all_queued);
 	RING_GETSIZE(struct fetch, fetch_ring, all_active);
 
-	FETCH_LOG(("queue_ring %i, fetch_ring %i", all_queued, all_active));
+	NSLOG(fetch, DEBUG,
+	      "queue_ring %i, fetch_ring %i",
+	      all_queued,
+	      all_active);
 	dump_rings();
 
 	while ((all_queued != 0) &&
@@ -253,12 +247,14 @@ static bool fetch_dispatch_jobs(void)
 	       fetch_choose_and_dispatch()) {
 			all_queued--;
 			all_active++;
-			FETCH_LOG(("%d queued, %d fetching",
-				   all_queued, all_active));
+			NSLOG(fetch, DEBUG,
+			      "%d queued, %d fetching",
+			      all_queued,
+			      all_active);
 	}
 
-	FETCH_LOG(("Fetch ring is now %d elements.", all_active));
-	FETCH_LOG(("Queue ring is now %d elements.", all_queued));
+	NSLOG(fetch, DEBUG, "Fetch ring is now %d elements.", all_active);
+	NSLOG(fetch, DEBUG, "Queue ring is now %d elements.", all_queued);
 
 	return (all_active > 0);
 }
@@ -268,7 +264,7 @@ static void fetcher_poll(void *unused)
 	int fetcherd;
 
 	if (fetch_dispatch_jobs()) {
-		FETCH_LOG(("Polling fetchers"));
+		NSLOG(fetch, DEBUG, "Polling fetchers");
 		for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
 			if (fetchers[fetcherd].refcount > 0) {
 				/* fetcher present */
@@ -277,7 +273,7 @@ static void fetcher_poll(void *unused)
 		}
 
 		/* schedule active fetchers to run again in 10ms */
-		guit->browser->schedule(SCHEDULE_TIME, fetcher_poll, NULL);
+		guit->misc->schedule(SCHEDULE_TIME, fetcher_poll, NULL);
 	}
 }
 
@@ -290,10 +286,12 @@ nserror fetcher_init(void)
 {
 	nserror ret;
 
+#ifdef WITH_CURL
 	ret = fetch_curl_register();
 	if (ret != NSERROR_OK) {
 		return ret;
 	}
+#endif
 
 	ret = fetch_data_register();
 	if (ret != NSERROR_OK) {
@@ -340,9 +338,10 @@ void fetcher_quit(void)
 			 * the reference count to allow the fetcher to
 			 * be stopped.
 			 */
-			LOG(("Fetcher for scheme %s still has %d active users at quit.",
-			     lwc_string_data(fetchers[fetcherd].scheme),
-			     fetchers[fetcherd].refcount));
+			NSLOG(fetch, INFO,
+			      "Fetcher for scheme %s still has %d active users at quit.",
+			      lwc_string_data(fetchers[fetcherd].scheme),
+			      fetchers[fetcherd].refcount);
 
 			fetchers[fetcherd].refcount = 1;
 		}
@@ -382,22 +381,22 @@ fetcher_add(lwc_string *scheme, const struct fetcher_operation_table *ops)
 }
 
 /* exported interface documented in content/fetch.h */
-nserror fetcher_fdset(fd_set *read_fd_set,
+nserror
+fetch_fdset(fd_set *read_fd_set,
 		      fd_set *write_fd_set,
 		      fd_set *except_fd_set,
 		      int *maxfd_out)
 {
-	CURLMcode code;
-	int maxfd;
+	int maxfd = -1;
 	int fetcherd; /* fetcher index */
 
 	if (!fetch_dispatch_jobs()) {
-		FETCH_LOG(("No jobs"));
+		NSLOG(fetch, DEBUG, "No jobs");
 		*maxfd_out = -1;
 		return NSERROR_OK;
 	}
 
-	FETCH_LOG(("Polling fetchers"));
+	NSLOG(fetch, DEBUG, "Polling fetchers");
 
 	for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
 		if (fetchers[fetcherd].refcount > 0) {
@@ -409,12 +408,19 @@ nserror fetcher_fdset(fd_set *read_fd_set,
 	FD_ZERO(read_fd_set);
 	FD_ZERO(write_fd_set);
 	FD_ZERO(except_fd_set);
-	code = curl_multi_fdset(fetch_curl_multi,
-				read_fd_set,
-				write_fd_set,
-				except_fd_set,
-				&maxfd);
-	assert(code == CURLM_OK);
+
+	for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
+		if ((fetchers[fetcherd].refcount > 0) &&
+		    (fetchers[fetcherd].ops.fdset != NULL)) {
+			/* fetcher present */
+			int fetcher_maxfd;
+			fetcher_maxfd = fetchers[fetcherd].ops.fdset(
+				fetchers[fetcherd].scheme, read_fd_set,
+				write_fd_set, except_fd_set);
+			if (fetcher_maxfd > maxfd)
+				maxfd = fetcher_maxfd;
+		}
+	}
 
 	if (maxfd >= 0) {
 		/* change the scheduled poll to happen is a 1000ms as
@@ -430,7 +436,7 @@ nserror fetcher_fdset(fd_set *read_fd_set,
 		 * select on. All the other fetchers continue to need
 		 * polling frequently.
 		 */
-		guit->browser->schedule(FDSET_TIMEOUT, fetcher_poll, NULL);
+		guit->misc->schedule(FDSET_TIMEOUT, fetcher_poll, NULL);
 	}
 
 	*maxfd_out = maxfd;
@@ -439,7 +445,7 @@ nserror fetcher_fdset(fd_set *read_fd_set,
 }
 
 /* exported interface documented in content/fetch.h */
-struct fetch *
+nserror
 fetch_start(nsurl *url,
 	    nsurl *referer,
 	    fetch_callback callback,
@@ -449,7 +455,8 @@ fetch_start(nsurl *url,
 	    const struct fetch_multipart_data *post_multipart,
 	    bool verifiable,
 	    bool downgrade_tls,
-	    const char *headers[])
+	    const char *headers[],
+	    struct fetch **fetch_out)
 {
 	struct fetch *fetch;
 	lwc_string *scheme;
@@ -457,7 +464,7 @@ fetch_start(nsurl *url,
 
 	fetch = malloc(sizeof (*fetch));
 	if (fetch == NULL) {
-		return NULL;
+		return NSERROR_NOMEM;
 	}
 
 	/* The URL we're fetching must have a scheme */
@@ -469,10 +476,10 @@ fetch_start(nsurl *url,
 	if (fetch->fetcherd == -1) {
 		lwc_string_unref(scheme);
 		free(fetch);
-		return NULL;
+		return NSERROR_NO_FETCH_HANDLER;
 	}
 
-	FETCH_LOG(("fetch %p, url '%s'", fetch, nsurl_access(url)));
+	NSLOG(fetch, DEBUG, "fetch %p, url '%s'", fetch, nsurl_access(url));
 
 	/* construct a new fetch structure */
 	fetch->callback = callback;
@@ -549,7 +556,11 @@ fetch_start(nsurl *url,
 
 		free(fetch);
 
-		return NULL;
+
+	/** \todo The fetchers setup should return nserror and that be
+	 * passed back rather than assuming a bad url
+	 */
+		return NSERROR_BAD_URL;
 	}
 
 	/* Rah, got it, so ref the fetcher. */
@@ -560,27 +571,32 @@ fetch_start(nsurl *url,
 
 	/* Ask the queue to run. */
 	if (fetch_dispatch_jobs()) {
-		FETCH_LOG(("scheduling poll"));
+		NSLOG(fetch, DEBUG, "scheduling poll");
 		/* schedule active fetchers to run again in 10ms */
-		guit->browser->schedule(10, fetcher_poll, NULL);
+		guit->misc->schedule(10, fetcher_poll, NULL);
 	}
 
-	return fetch;
+	*fetch_out = fetch;
+	return NSERROR_OK;
 }
 
 /* exported interface documented in content/fetch.h */
 void fetch_abort(struct fetch *f)
 {
 	assert(f);
-	FETCH_LOG(("fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
-	     nsurl_access(f->url)));
+	NSLOG(fetch, DEBUG,
+	      "fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
+	     nsurl_access(f->url));
 	fetchers[f->fetcherd].ops.abort(f->fetcher_handle);
 }
 
 /* exported interface documented in content/fetch.h */
 void fetch_free(struct fetch *f)
 {
-	FETCH_LOG(("Freeing fetch %p, fetcher %p", f, f->fetcher_handle));
+	NSLOG(fetch, DEBUG,
+	      "Freeing fetch %p, fetcher %p",
+	      f,
+	      f->fetcher_handle);
 
 	fetchers[f->fetcherd].ops.free(f->fetcher_handle);
 
@@ -630,13 +646,6 @@ long fetch_http_code(struct fetch *fetch)
 	return fetch->http_code;
 }
 
-/* exported interface documented in content/fetch.h */
-bool fetch_get_verifiable(struct fetch *fetch)
-{
-	assert(fetch);
-
-	return fetch->verifiable;
-}
 
 /* exported interface documented in content/fetch.h */
 struct fetch_multipart_data *
@@ -713,7 +722,8 @@ void fetch_multipart_data_destroy(struct fetch_multipart_data *list)
 		free(list->name);
 		free(list->value);
 		if (list->file) {
-			FETCH_LOG(("Freeing rawfile: %s", list->rawfile));
+			NSLOG(fetch, DEBUG,
+			      "Freeing rawfile: %s", list->rawfile);
 			free(list->rawfile);
 		}
 		free(list);
@@ -731,8 +741,13 @@ fetch_send_callback(const fetch_msg *msg, struct fetch *fetch)
 /* exported interface documented in content/fetch.h */
 void fetch_remove_from_queues(struct fetch *fetch)
 {
-	FETCH_LOG(("Fetch %p, fetcher %p can be freed",
-		   fetch, fetch->fetcher_handle));
+	int all_active;
+	int all_queued;
+
+	NSLOG(fetch, DEBUG,
+	      "Fetch %p, fetcher %p can be freed",
+	      fetch,
+	      fetch->fetcher_handle);
 
 	/* Go ahead and free the fetch properly now */
 	if (fetch->fetch_is_active) {
@@ -741,24 +756,19 @@ void fetch_remove_from_queues(struct fetch *fetch)
 		RING_REMOVE(queue_ring, fetch);
 	}
 
-#ifdef DEBUG_FETCH_VERBOSE
-	int all_active;
-	int all_queued;
 
 	RING_GETSIZE(struct fetch, fetch_ring, all_active);
 	RING_GETSIZE(struct fetch, queue_ring, all_queued);
 
-	LOG(("Fetch ring is now %d elements.", all_active));
-
-	LOG(("Queue ring is now %d elements.", all_queued));
-#endif
+	NSLOG(fetch, DEBUG, "Fetch ring is now %d elements.", all_active);
+	NSLOG(fetch, DEBUG, "Queue ring is now %d elements.", all_queued);
 }
 
 
 /* exported interface documented in content/fetch.h */
 void fetch_set_http_code(struct fetch *fetch, long http_code)
 {
-	FETCH_LOG(("Setting HTTP code to %ld", http_code));
+	NSLOG(fetch, DEBUG, "Setting HTTP code to %ld", http_code);
 
 	fetch->http_code = http_code;
 }
